# Copyright (c) 2015, Yahoo Inc.
# Copyrights licensed under the New BSD License
# See the accompanying LICENSE.txt file for terms.
"""
Tests for `redislite.patch` module.
"""
from __future__ import print_function
import getpass
import inspect
import logging
import os
import shutil
import tempfile
import unittest
import psutil
import redis
import redislite
import redislite.patch


logger = logging.getLogger(__name__)


# noinspection PyPep8Naming
class TestRedislitePatch(unittest.TestCase):

    class logger(object):
        @staticmethod
        def debug(*args, **kwargs):
            new_args = list(args)
            new_args[0] = str(inspect.stack()[1][3]) + ': ' + new_args[0]
            args=tuple(new_args)
            return logger.debug(*args, **kwargs)

    def _redis_server_processes(self):
        """
        Return count of process objects for all redis-server processes started
        by the current user.
        :return:
        """
        processes = []
        for proc in psutil.process_iter():
            try:
                if proc.name() == 'redis-server':
                    if proc.username() == getpass.getuser():
                        processes.append(proc)
            except psutil.NoSuchProcess:
                pass

        return len(processes)

    def _log_redis_pid(self, connection):
        logger.debug(
            '%s: r.pid: %d', str(inspect.stack()[1][3]), connection.pid
        )

    def setUp(self):
        self.tempdir = tempfile.mkdtemp()
        logger.info(
            'Start Child processes: %d', self._redis_server_processes()
        )

    def tearDown(self):
        logger.info(
            'End Child processes: %d', self._redis_server_processes()
        )
        if os.path.exists(self.tempdir) and 'tmp' in self.tempdir:
            shutil.rmtree(self.tempdir)
            self.tempdir = None

    def test_redislite_patch_redis_Redis(self):
        redislite.patch.patch_redis_Redis()
        r = redis.Redis()
        self._log_redis_pid(r)
        self.assertIsInstance(r.pid, int)    # Should have a redislite pid
        redislite.patch.unpatch_redis_Redis()

    def test_redislite_patch_redis_StrictRedis(self):
        redislite.patch.patch_redis_StrictRedis()
        r = redis.StrictRedis()
        self._log_redis_pid(r)
        self.assertIsInstance(r.pid, int)    # Should have a redislite pid
        redislite.patch.unpatch_redis_StrictRedis()

    # noinspection PyUnusedLocal
    def test_redislite_patch_redis(self):
        redislite.patch.patch_redis()
        r = redis.Redis()
        self._log_redis_pid(r)
        self.assertIsInstance(r.pid, int)    # Should have a redislite pid
        s = redis.StrictRedis()
        self._log_redis_pid(s)
        self.assertIsInstance(s.pid, int)    # Should have a redislite pid
        redislite.patch.unpatch_redis()

    def test_redislite_double_patch_redis(self):
        original_redis = redis.Redis
        redislite.patch.patch_redis()
        self.assertNotEqual(original_redis, redis.Redis)
        redislite.patch.patch_redis()
        self.assertNotEqual(original_redis, redis.Redis)
        redislite.patch.unpatch_redis()
        self.assertEqual(original_redis, redis.Redis)

    def test_redislite_patch_redis_with_dbfile(self):
        dbfilename = os.path.join(
            self.tempdir,
            'test_redislite_patch_redis_with_dbfile.db'
        )
        if os.path.exists(dbfilename):
            os.remove(dbfilename)
        redislite.patch.patch_redis(dbfilename)
        r = redis.Redis()
        self._log_redis_pid(r)
        self.assertIsInstance(r.pid, int)    # Should have a redislite pid
        s = redis.Redis()
        self._log_redis_pid(s)
        self.assertIsInstance(r.pid, int)    # Should have a redislite pid

        # Both instances should be talking to the same redis server
        self.assertEqual(r.pid, s.pid)
        redislite.patch.unpatch_redis()

        r._cleanup()
        s._cleanup()
        if os.path.exists(dbfilename):
            os.remove(dbfilename)

    def test_redislite_patch_redis_with_dbfile_in_cwd(self):
        dbfilename = 'test_redislite_patch_redis_with_dbfile_in_cwd.db'
        if os.path.exists(dbfilename):
            os.remove(dbfilename)
        redislite.patch.patch_redis(dbfilename)
        r = redis.Redis()
        self._log_redis_pid(r)
        self.assertIsInstance(r.pid, int)  # Should have a redislite pid
        s = redis.Redis()
        self._log_redis_pid(s)
        self.assertIsInstance(r.pid, int)  # Should have a redislite pid

        # Both instances should be talking to the same redis server
        self.assertEqual(r.pid, s.pid)
        redislite.patch.unpatch_redis()

        r._cleanup()
        s._cleanup()
        if os.path.exists(dbfilename):
            os.remove(dbfilename)

    def test_redislite_patch_strictredis_with_dbfile(self):
        dbfilename = os.path.join(
            self.tempdir,
            'test_redislite_patch_redis_with_short_dbfile.db'
        )
        if os.path.exists(dbfilename):
            os.remove(dbfilename)
        redislite.patch.patch_redis_StrictRedis(dbfilename)
        r = redis.StrictRedis()
        self._log_redis_pid(r)
        self.assertIsInstance(r.pid, int)    # Should have a redislite pid
        s = redis.StrictRedis()
        self._log_redis_pid(s)
        self.assertIsInstance(r.pid, int)    # Should have a redislite pid

        # Both instances should be talking to the same redis server
        self.assertEqual(r.pid, s.pid)
        redislite.patch.unpatch_redis_StrictRedis()
        r._cleanup()
        s._cleanup()
        if os.path.exists(dbfilename):
            os.remove(dbfilename)


if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG)
    test_suite = unittest.TestLoader().loadTestsFromTestCase(TestRedislitePatch)
    unittest.TextTestRunner(verbosity=2).run(test_suite)



# Copyright (c) 2015, Yahoo Inc.
# Copyrights licensed under the New BSD License
# See the accompanying LICENSE.txt file for terms.
"""
Tests for `redislite` module (__init__.py).
"""
from __future__ import print_function
import logging
import redislite
import redislite.patch
import unittest


logger = logging.getLogger(__name__)


# noinspection PyPep8Naming
class TestRedisliteModule(unittest.TestCase):
    def test_metadata(self):
        self.assertIsInstance(redislite.__version__, str)
        self.assertIsInstance(redislite.__git_version__, str)
        self.assertIsInstance(redislite.__git_origin__, str)
        self.assertIsInstance(redislite.__git_branch__, str)
        self.assertIsInstance(redislite.__git_hash__, str)
        self.assertIsInstance(redislite.__redis_server_info__, dict)
        self.assertIsInstance(redislite.__redis_executable__, str)


if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG)
    test_suite = unittest.TestLoader().loadTestsFromTestCase(
        TestRedisliteModule
    )
    unittest.TextTestRunner(verbosity=2).run(test_suite)



# Copyright (c) 2015, Yahoo Inc.
# Copyrights licensed under the New BSD License
# See the accompanying LICENSE.txt file for terms.
"""
Tests for `redislite.debug` module.
"""
from __future__ import print_function
import logging
import redislite
import unittest


logger = logging.getLogger(__name__)


# noinspection PyPep8Naming
class TestRedisliteDebug(unittest.TestCase):

    def test_debug(self):
        import redislite.debug
        redislite.debug.print_debug_info()


if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG)
    test_suite = unittest.TestLoader().loadTestsFromTestCase(TestRedisliteDebug)
    unittest.TextTestRunner(verbosity=2).run(test_suite)



# Copyright (c) 2015, Yahoo Inc.
# Copyrights licensed under the New BSD License
# See the accompanying LICENSE.txt file for terms.
"""
test_redislite
----------------------------------

Tests for `redislite` module.
"""
from __future__ import print_function
import getpass
import inspect
import logging
import os
import psutil
import redislite
import redislite.configuration
import tempfile
import shutil
import unittest


logger = logging.getLogger(__name__)


# noinspection PyPep8Naming
class TestRedisliteConfiguration(unittest.TestCase):

    class logger(object):
        @staticmethod
        def debug(*args, **kwargs):
            new_args = list(args)
            new_args[0] = str(inspect.stack()[1][3]) + ': ' + new_args[0]
            args=tuple(new_args)
            return logger.debug(*args, **kwargs)

    def _redis_server_processes(self):
        """
        Return count of process objects for all redis-server processes started
        by the current user.
        :return:
        """
        processes = []
        for proc in psutil.process_iter():
            try:
                if proc.name() == 'redis-server':
                    if proc.username() == getpass.getuser():
                        processes.append(proc)
            except psutil.NoSuchProcess:
                pass

        return len(processes)

    def _log_redis_pid(self, connection):
        logger.debug(
            '%s: r.pid: %d', str(inspect.stack()[1][3]), connection.pid
        )

    def setUp(self):
        self.tempdir = tempfile.mkdtemp()
        logger.info(
            'Start Child processes: %d', self._redis_server_processes()
        )

    def tearDown(self):
        if os.path.exists(self.tempdir) and 'tmp' in self.tempdir:
            shutil.rmtree(self.tempdir)
            self.tempdir = None
        logger.info(
            'End Child processes: %d', self._redis_server_processes()
        )

    def test_debug(self):
        import redislite.debug
        redislite.debug.print_debug_info()

    def test_configuration_config_line(self):
        result = redislite.configuration.config_line(
            'dbfilename', 'test file.db'
        )
        self.assertEqual("dbfilename 'test file.db'", result)

    def test_configuration_config(self):
        result = redislite.configuration.config()
        self.assertIn('\ndaemonize yes', result)

    def test_configuration_modify_defaults(self):
        result = redislite.configuration.config(daemonize="no")
        self.assertIn('\ndaemonize no', result)

        # ensure the global defaults are not modified
        self.assertEquals(
            redislite.configuration.DEFAULT_REDIS_SETTINGS["daemonize"], "yes"
        )


    def test_configuration_settings(self):
        result = redislite.configuration.settings()
        del result['save']  # Save is a list which doesn't work with set
        result_set = set(result.items())
        expected_subset = set(
            {
                'dbdir': './',
                'daemonize': 'yes',
                'unixsocketperm': '700',
                'timeout': '0',
                'dbfilename': 'redis.db'
            }.items()
        )
        self.assertTrue(
            expected_subset.issubset(result_set)
        )


    def test_configuration_config_db(self):
        pidfile = os.path.join(self.tempdir, 'test.pid')
        unixsocket = os.path.join(self.tempdir, 'redis.socket')
        result = redislite.configuration.config(
                pidfile=pidfile,
                unixsocket= unixsocket,
                dbdir=self.tempdir,
                dbfilename='test.db',
        )

        self.assertIn('\ndaemonize yes', result)
        self.assertIn('\npidfile ' + repr(pidfile), result)
        self.assertIn("\ndbfilename 'test.db'", result)

    def test_configuration_config_db_with_space(self):
        pidfile = os.path.join(self.tempdir, 'test space.pid')
        unixsocket = os.path.join(self.tempdir, 'test space.socket')
        result = redislite.configuration.config(
            pidfile=pidfile,
            unixsocket=unixsocket,
            dbdir=self.tempdir,
            dbfilename='test space.db',
        )

        self.assertIn('\ndaemonize yes', result)
        self.assertIn('\npidfile ' + repr(pidfile), result)
        self.assertIn("\ndbfilename 'test space.db'", result)

    def test_configuration_config_slave(self):
        result = redislite.configuration.config(
                pidfile='/var/run/redislite/test.pid',
                unixsocket='/var/run/redislite/redis.socket',
                dbdir=os.getcwd(),
                dbfilename='test.db',
                slaveof='localhost 6397'
        )
        self.assertIn('slaveof localhost 6397', result)


if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG)
    test_suite = unittest.TestLoader().loadTestsFromTestCase(
        TestRedisliteConfiguration
    )
    unittest.TextTestRunner(verbosity=2).run(test_suite)



# Copyright (c) 2015, Yahoo Inc.
# Copyrights licensed under the New BSD License
# See the accompanying LICENSE.txt file for terms.
"""
Redislite Debug Utilities

This module contains utility functions useful for troubleshooting redislite.

This module can be run from the command line using the following command::

    python -m redislite.debug


This will output information like the following::

    $ python -m redislite.debug
    Redislite debug information:
        Version: 1.0.171
        Module Path: /tmp/redislite/lib/python3.4/site-packages/redislite

        Installed Redis Server:
            Redis Executable: /tmp/redislite/lib/python3.4/site-packages/redislite/bin/redis-server
            build = 3a2b5dab9c14cd5e
            sha = 4657e47d:1
            bits = 64
            v = 2.8.17
            malloc = libc

        Found redis-server: /tmp/redislite/lib/python3.4/site-packages/redislite/bin/redis-server
            v = 2.8.17
            sha = 4657e47d:1
            malloc = libc
            bits = 64
            build = 3a2b5dab9c14cd5e

        Source Code Information
            Git Source URL: https://github.com/yahoo/redislite/tree/2ebd1b4d9c9ad41c78e8048fda3c69d2917c0348
            Git Hash: 2ebd1b4d9c9ad41c78e8048fda3c69d2917c0348
            Git Version: 1.0.171
            Git Origin: https://github.com/yahoo/redislite.git
            Git Branch: master


When run from the command line this will print a dump of information about
the module and it's build information.
"""
from __future__ import print_function
from distutils.spawn import find_executable
import os
from .__init__ import __version__, __git_version__, __source_url__, \
    __git_hash__, __git_origin__, __git_branch__, __redis_server_info__, \
    __redis_executable__


def debug_info_list():
    """
    Return a list with the debug information
    :return:
    """
    info = []
    redis_server = find_executable('redis-server')
    if __redis_executable__:  # pragma: no cover
        redis_server = __redis_executable__
    info.append("Redislite debug information:")
    info.append('\tVersion: %s' % __version__)
    info.append('\tModule Path: %s' % os.path.dirname(__file__))
    info.append('\n\tInstalled Redis Server:')
    info.append('\t\tRedis Executable: %s' % redis_server)
    for key, value in __redis_server_info__.items():  # pragma: no cover
        info.append('\t\t%s = %s' % (key, value))
    info.append('\n\tFound redis-server: %s' % redis_server)
    for item in os.popen('%s --version' % redis_server).read().strip().split():
        if '=' in item:
            key, value = item.split('=')
            info.append('\t\t%s = %s' % (key, value))
    info.append('')
    info.append('\tSource Code Information')
    if __git_version__:  # pragma: no cover
        info.append('\t\tGit Source URL: %s' % __source_url__)
        info.append('\t\tGit Hash: %s' % __git_hash__)
        info.append('\t\tGit Version: %s' % __git_version__)
        info.append('\t\tGit Origin: %s' % __git_origin__)
        info.append('\t\tGit Branch: %s' % __git_branch__)
    return info


def debug_info():
    """
    Return a multi-line string with the debug information
    :return:
    """
    return os.linesep.join(debug_info_list())


def print_debug_info():
    """
    Display information about the redislite build, and redis-server on stdout.
    :return:
    """
    print(debug_info())


if __name__ == '__main__':  # pragma: no cover
    print_debug_info()



# Copyright (c) 2015, Yahoo Inc.
# Copyrights licensed under the New BSD License
# See the accompanying LICENSE.txt file for terms.
"""
Tests for `redislite.client` module.
"""
from __future__ import print_function
import getpass
import inspect
import logging
import os
import psutil
import redislite
import shutil
import stat
import tempfile
import time
import unittest
from redis.exceptions import ConnectionError


logger = logging.getLogger(__name__)


# noinspection PyPep8Naming
class TestRedisliteClient(unittest.TestCase):

    class logger(object):
        @staticmethod
        def debug(*args, **kwargs):
            new_args = list(args)
            new_args[0] = str(inspect.stack()[1][3]) + ': ' + new_args[0]
            args=tuple(new_args)
            return logger.debug(*args, **kwargs)

    def _redis_server_processes(self):
        """
        Return count of process objects for all redis-server processes started
        by the current user.
        :return:
        """
        processes = []
        for proc in psutil.process_iter():
            try:
                if proc.name() == 'redis-server':
                    if proc.username() == getpass.getuser():
                        processes.append(proc)
            except psutil.NoSuchProcess:
                pass

        return len(processes)

    def _log_redis_log(self, connection):
        if os.path.exists(connection.logfile):
            with open(connection.logfile) as f:
                print(f.read())

    def _log_redis_pid(self, connection):
        print(
            f'{str(inspect.stack()[1][3])}: r.pid: {connection.pid}'
        )

    def setUp(self):
        logger.info(
            'Start Child processes: %d', self._redis_server_processes()
        )

    def tearDown(self):
        logger.info(
            'End Child processes: %d', self._redis_server_processes()
        )

    def test_redislite_Redis(self):
        r = redislite.Redis()
        self._log_redis_pid(r)
        r.set(b'key', b'value')
        time.sleep(1)
        value = r.get(b'key')
        r.redis_log_tail(lines=0)
        self.assertIsInstance(value, bytes)
        self.assertEqual(value, b'value')

    def test_redislite_Redis_multiple(self):
        r1 = redislite.Redis()
        r2 = redislite.Redis()
        self._log_redis_pid(r1)
        self._log_redis_pid(r2)

        r1.set('key', 'value')
        r2.set('key2', 'value2')
        self.assertTrue(len(r1.keys()), 1)
        self.assertTrue(len(r2.keys()), 1)
        result = r1.get('key').decode(encoding='UTF-8')
        self.assertEqual(result, 'value')
        result = r2.get('key2').decode(encoding='UTF-8')
        self.assertEqual(result, 'value2')

    def test_redislite_Redis_with_db_file(self):
        temp_dir = tempfile.mkdtemp()
        filename = os.path.join(temp_dir, 'redis.db')
        self.assertFalse(os.path.exists(filename))
        r = redislite.Redis(filename)
        self._log_redis_pid(r)
        r.set('key', 'value')
        result = r.get('key').decode(encoding='UTF-8')
        self.assertEqual(result, 'value')
        r.save()
        r._cleanup()
        self.assertTrue(os.path.exists(filename))
        shutil.rmtree(temp_dir)

    def test_redislite_Redis_with_db_file_keyword(self):
        temp_dir = tempfile.mkdtemp()
        filename = os.path.join(temp_dir, 'redis.db')
        self.assertFalse(os.path.exists(filename))
        r = redislite.Redis(dbfilename=filename)
        self._log_redis_pid(r)
        r.set('key', 'value')
        result = r.get('key').decode(encoding='UTF-8')
        self.assertEqual(result, 'value')
        r.save()
        r._cleanup()
        self.assertTrue(os.path.exists(filename))
        shutil.rmtree(temp_dir)

    def test_redislite_Redis_multiple_connections(self):
        # Generate a new redis server
        r = redislite.Redis()
        self._log_redis_pid(r)

        # Pass the first server's db to get a second connection
        s = redislite.Redis(r.db)
        self._log_redis_pid(s)
        r.set('key', 'value')
        result = s.get('key').decode(encoding='UTF-8')
        self.assertEqual(result, 'value')

        # Both objects should be using the same redis process.
        self.assertEqual(r.pid, s.pid)

    def test_redislite_Redis_cleanup(self):
        r = redislite.Redis()
        self._log_redis_pid(r)
        r._cleanup()
        self.assertIsNone(r.socket_file)

    def test_redislite_Redis_create_redis_directory_tree(self):
        r = redislite.Redis()
        self._log_redis_pid(r)
        r._create_redis_directory_tree()
        self.assertTrue(r.redis_dir)
        self.assertTrue(os.path.exists(r.redis_dir))
        self.assertTrue(os.path.exists(r.pidfile))
        self.assertTrue(os.path.exists(r.socket_file))
        r._cleanup()

    def test_redislite_redis_custom_socket_file_local_directory(self):
        """
        Test creating a redis instance with a specified socket filename
        :return:
        """
        socket_file_name = 'test.socket'
        full_socket_file_name = os.path.join(os.getcwd(), socket_file_name)
        r = redislite.Redis(unix_socket_path=socket_file_name)
        self._log_redis_pid(r)
        r.redis_log_tail()
        self.assertEqual(r.socket_file, full_socket_file_name)
        print(os.listdir('.'))
        mode = os.stat(socket_file_name).st_mode
        isSocket = stat.S_ISSOCK(mode)
        self.assertTrue(isSocket)
        r._cleanup()

    def test_redislite_redis_custom_socket_file(self):
        """
        Test creating a redis instance with a specified socket filename
        :return:
        """
        socket_file_name = '/tmp/test.socket'
        r = redislite.Redis(unix_socket_path=socket_file_name)
        self._log_redis_pid(r)
        self.assertEqual(r.socket_file, socket_file_name)
        print(os.listdir('.'))
        mode = os.stat(socket_file_name).st_mode
        isSocket = stat.S_ISSOCK(mode)
        self.assertTrue(isSocket)
        r._cleanup()

    def test_redislite_Redis_pid(self):
        r = redislite.Redis()
        self._log_redis_pid(r)
        self.assertTrue(r.pidfile)
        self.assertGreater(r.pid, 0)

    def test_redislite_db_file_cwd_args(self):
        test_db = 'test_unit_redis.db'
        if os.path.exists(test_db):
            os.remove(test_db)
        r = redislite.Redis(test_db)
        self._log_redis_pid(r)
        r.set('key', 'value')
        r.save()
        self.assertTrue(os.path.exists(test_db))
        os.remove(test_db)

    def test_redislite_db_file_cwd_kw(self):
        test_db = 'test_unit_redis.db'
        if os.path.exists(test_db):
            os.remove(test_db)
        r = redislite.Redis(dbfilename=test_db)
        self._log_redis_pid(r)
        r.set('key', 'value')
        r.save()
        self.assertTrue(os.path.exists(test_db))
        os.remove(test_db)

    def test_is_redis_running_no_pidfile(self):
        r = redislite.Redis()
        self.assertTrue(r._is_redis_running(), 'Redis server should be running')

        print("Shutting down the server the hard way", flush=True)
        r.shutdown()

        print('Checking if the server is running', flush=True)
        result = r._is_redis_running()
        self.assertFalse(result, 'Redis server should not be running')

    def test_refcount_cleanup(self):
        self.logger.debug('Setting up 2 connections to a single redis server.')
        r = redislite.Redis()
        self._log_redis_pid(r)
        s = redislite.Redis(r.db)
        self._log_redis_pid(s)

        pid = r.pid
        redis_dir = r.redis_dir

        self.logger.debug('Making sure the redis-server is running')
        p = psutil.Process(pid)
        self.assertTrue(p.is_running())

        self.logger.debug(
            'Shuting down the first instance, redis-server should remain '
            'running due to the other connection.'
        )
        r._cleanup()

        self._log_redis_log(r)
        self.assertTrue(
            os.path.exists(redis_dir),
            msg='Shutting down the server removed the temporary directory'
        )
        self.assertEqual(
            s.pid, pid,
            msg='Redis server shutdown with active connection'
        )

        p = psutil.Process(pid)
        self.assertTrue(
            p.is_running(),
            msg='Redis server shutdown with active connection'
        )

        self.logger.debug(
            'Shutting down  the second instance, the redis-server should '
            'be gone after the connection terminates'
        )
        self.logger.debug(
            'Second connection count is: %s', s._connection_count()
        )
        s._cleanup()
        with self.assertRaises(psutil.NoSuchProcess, msg=f'Redis cleanup method did not terminate pid {pid}'):
            p = psutil.Process(pid)

    def test_connection_count(self):
        r = redislite.Redis()
        self._log_redis_pid(r)
        self.assertEqual(r._connection_count(), 1)

    def test_connection_count_multiple(self):
        r = redislite.Redis()
        self._log_redis_pid(r)
        s = redislite.Redis(r.db)
        self.assertEqual(r._connection_count(), 2)

    def test_connection_fallthrough(self):
        """
        Create a connection with an argument that will cause redislite
        to pass it to the redis module.  This should generate an exception
        since there is no redis server running on the port.
        :return:
        """
        with self.assertRaises(ConnectionError):
            redislite.Redis(port=1).keys()

    def test_shutdown_race_condition(self):
        """
        Test that there is no race condition when a shutdown is run after
        a large data operation.
        """
        if os.path.exists('bug.redis'):
            os.remove('bug.redis')
        db = redislite.StrictRedis('bug.redis')
        for key in range(10000):
            db.hset("h1", os.urandom(32), " " * 65536)
        db.save()
        db._cleanup()
        del db
        db = redislite.StrictRedis('bug.redis')
        db._cleanup()
        if os.path.exists('bug.redis'):
            os.remove('bug.redis')

    def test_redis_log_attribute(self):
        r = redislite.StrictRedis()
        self.assertIn('The server is now ready to accept connections', r.redis_log)

    def test_redis_log_tail(self):
        r = redislite.StrictRedis()
        lines = r.redis_log_tail(2)
        self.assertIsInstance(lines, list)
        self.assertEqual(len(lines), 2)

    def test_redis_log_many_lines(self):
        r = redislite.StrictRedis()
        lines = r.redis_log_tail(lines=99999)
        self.assertIsInstance(lines, list)

    def test_redis_log_small_chunks(self):
        r = redislite.StrictRedis()
        lines = r.redis_log_tail(4, width=20)
        self.assertIsInstance(lines, list)
        self.assertEqual(len(lines), 4)

    def test_redis_log_tail_no_log(self):
        r = redislite.StrictRedis()
        if os.path.exists(r.logfile):
            os.remove(r.logfile)
        lines = r.redis_log_tail()
        self.assertEqual(lines, [])

    def test_redis_log_tail_empty_log(self):
        r = redislite.StrictRedis()
        with open(r.logfile, 'w'):
            pass
        lines = r.redis_log_tail()
        self.assertEqual(lines, [])


if __name__ == '__main__':
    logging.basicConfig(level=logging.DEBUG)
    test_suite = unittest.TestLoader().loadTestsFromTestCase(
        TestRedisliteClient
    )
    unittest.TextTestRunner(verbosity=2).run(test_suite)



# Copyright (c) 2015, Yahoo Inc.
# Copyrights licensed under the New BSD License
# See the accompanying LICENSE.txt file for terms.
"""
This module contains functions to generate a redis configuration from a
configuration template.
"""
import logging
from copy import copy


logger = logging.getLogger(__name__)  # pylint: disable=C0103


DEFAULT_REDIS_SETTINGS = {
    'activerehashing': 'yes',
    'aof-rewrite-incremental-fsync': 'yes',
    'appendonly': 'no',
    'appendfilename': 'appendonly.aof',
    'appendfsync': 'everysec',
    'aof-load-truncated': 'yes',
    'auto-aof-rewrite-percentage': '100',
    'auto-aof-rewrite-min-size': '64mb',
    'bind': None,
    'daemonize': 'yes',
    'databases': '16',
    'dbdir': './',
    'dbfilename': 'redis.db',
    'hash-max-ziplist-entries': '512',
    'hash-max-ziplist-value': '64',
    'hll-sparse-max-bytes': '3000',
    'hz': '10',
    'list-max-ziplist-entries': '512',
    'list-max-ziplist-value': '64',
    'loglevel': 'notice',
    'logfile': 'redis.log',
    'lua-time-limit': '5000',
    'pidfile': '/var/run/redislite/redis.pid',
    'port': '0',
    'save': ['900 1', '300 100', '60 200', '15 1000'],
    'stop-writes-on-bgsave-error': 'yes',
    'tcp-backlog': '511',
    'tcp-keepalive': '0',
    'rdbcompression': 'yes',
    'rdbchecksum': 'yes',
    'slave-serve-stale-data': 'yes',
    'slave-read-only': 'yes',
    'repl-disable-tcp-nodelay': 'no',
    'slave-priority': '100',
    'no-appendfsync-on-rewrite': 'no',
    'slowlog-log-slower-than': '10000',
    'slowlog-max-len': '128',
    'latency-monitor-threshold': '0',
    'notify-keyspace-events': '""',
    'set-max-intset-entries': '512',
    'timeout': '0',
    'unixsocket': '/var/run/redislite/redis.socket',
    'unixsocketperm': '700',
    'zset-max-ziplist-entries': '128',
    'zset-max-ziplist-value': '64',
}


def settings(**kwargs):
    """
    Get config settings based on the defaults and the arguments passed

    Parameters
    ----------
    **kwargs
        Redis server arguments, the keyword is the setting, the value is the
        value.
        If the value is of type list, the setting will be  set multiple times
        once with each value.
        If the value is None, the setting will be removed from the default
        settings if it exists.

    Returns
    -------
    dict
        Dictionary containing redis server settings, with the key being the
        setting name.  The values are the settings.  If the value is a list
        the setting will be repeated with each specified value.
    """
    new_settings = copy(DEFAULT_REDIS_SETTINGS)
    new_settings.update(kwargs)

    return new_settings


def config_line(setting, value):
    """
    Generate a single configuration line based on the setting and value

    Parameters
    ----------
    setting : str
        The configuration setting

    value : str
        The value for the configuration setting

    Returns
    -------
    str
        The configuration line based on the setting and value
    """
    if setting in [
        'appendfilename', 'dbfilename', 'dbdir', 'dir', 'pidfile', 'unixsocket'
    ]:
        value = repr(value)
    return '{setting} {value}'.format(setting=setting, value=value)


def config(**kwargs):
    """
    Generate a redis configuration file based on the passed arguments

    Returns
    -------
    str
        Redis server configuration
    """
    # Get our settings
    config_dict = settings(**kwargs)
    config_dict['dir'] = config_dict['dbdir']
    del config_dict['dbdir']

    configuration = ''
    keys = list(config_dict.keys())
    keys.sort()
    for key in keys:
        if config_dict[key]:
            if isinstance(config_dict[key], list):
                for item in config_dict[key]:
                    configuration += config_line(
                        setting=key, value=item
                    ) + '\n'
            else:
                configuration += config_line(
                    setting=key, value=config_dict[key]
                ) + '\n'
        else:
            del config_dict[key]
    logger.debug('Using configuration: %s', configuration)
    return configuration



#!/usr/bin/env python
# Copyright (c) 2015, Yahoo Inc.
# Copyrights licensed under the New BSD License
# See the accompanying LICENSE.txt file for terms.
from __future__ import print_function
import os
import json
import logging
import pathlib
import subprocess

from setuptools import setup
from setuptools.command.install import install
import shutil
import sys
import urllib.request
import tarfile
import tempfile
from distutils.command.build import build
from distutils.core import Extension
import distutils.util
from subprocess import call, check_output, CalledProcessError


logger = logging.getLogger(__name__)

UNSUPPORTED_PLATFORMS = ['win32', 'win64']
METADATA_FILENAME = 'redislite/package_metadata.json'
BASEPATH = os.path.dirname(os.path.abspath(__file__))
REDIS_PATH = os.path.join(BASEPATH, 'redis.submodule')
REDIS_SERVER_METADATA = {}
REDIS_VERSION = os.environ.get('REDIS_VERSION', '6.2.7')
REDIS_URL = f'http://download.redis.io/releases/redis-{REDIS_VERSION}.tar.gz'
install_scripts = ''
try:
    VERSION = check_output(['meta', 'get', 'package.version']).decode(errors='ignore')
except (subprocess.CalledProcessError, FileNotFoundError):
    VERSION = REDIS_VERSION


def download_redis_submodule():
    if pathlib.Path(REDIS_PATH).exists():
        shutil.rmtree(REDIS_PATH)
    with tempfile.TemporaryDirectory() as tempdir:
        print(f'Downloading {REDIS_URL} to temp directory {tempdir}')
        ftpstream = urllib.request.urlopen(REDIS_URL)
        tf = tarfile.open(fileobj=ftpstream, mode="r|gz")
        directory = tf.next().name

        print(f'Extracting archive {directory}')
        tf.extractall(tempdir)

        print(f'Moving {os.path.join(tempdir, directory)} -> redis.submodule')
        shutil.move(os.path.join(tempdir, directory), 'redis.submodule')

        # print('Updating jemalloc')
        # os.system('(cd redis.submodule;./deps/update-jemalloc.sh 4.0.4)')


class BuildRedis(build):
    global REDIS_SERVER_METADATA

    def run(self):
        # run original build code
        build.run(self)

        # build Redis
        logger.debug('Running build_redis')

        os.environ['CC'] = 'gcc'
        os.environ['PREFIX'] = REDIS_PATH
        cmd = [
            'make',
            'MALLOC=libc',
            'V=' + str(self.verbose),
        ]

        targets = ['install']
        cmd.extend(targets)

        target_files = [
            os.path.join(REDIS_PATH, 'bin/redis-server'),
            os.path.join(REDIS_PATH, 'bin/redis-cli'),
        ]

        def _compile():
            print('*' * 80)
            print(os.getcwd())
            call(cmd, cwd=REDIS_PATH)
            print('*' * 80)

        self.execute(_compile, [], 'compiling redis')

        # copy resulting tool to script folder
        self.mkpath(self.build_scripts)

        if not self.dry_run:
            for target in target_files:
                logger.debug('copy: %s -> %s', target, self.build_scripts)
                self.copy_file(target, self.build_scripts)


class InstallRedis(install):
    build_scripts = None

    def initialize_options(self):
        install.initialize_options(self)

    def finalize_options(self):
        install.finalize_options(self)
        self.set_undefined_options('build', ('build_scripts', 'build_scripts'))

    def run(self):
        global install_scripts
        # run original install code
        install.run(self)

        # install Redis executables
        logger.debug(
            'running InstallRedis %s -> %s', self.build_lib, self.install_lib
        )
        self.copy_tree(self.build_lib, self.install_lib)
        module_bin = os.path.join(self.install_lib, 'redislite/bin')
        if not os.path.exists(module_bin):
            os.makedirs(module_bin, 0o0755)
        self.copy_tree(self.build_scripts, module_bin)
        logger.debug(
            'running InstallRedis %s -> %s',
            self.build_scripts, self.install_scripts
        )
        self.copy_tree(self.build_scripts, self.install_scripts)

        install_scripts = self.install_scripts
        print('install_scripts: %s' % install_scripts)
        md_file = os.path.join(
            self.install_lib, 'redislite/package_metadata.json'
        )
        if os.path.exists(md_file):
            with open(md_file) as fh:
                md = json.load(fh)
                if os.path.exists(os.path.join(module_bin, 'redis-server')):
                    md['redis_bin'] = os.path.join(module_bin, 'redis-server')
                else:
                    md['redis_bin'] = os.path.join(
                        install_scripts, 'redis-server'
                    )
            # Store the redis-server --version output for later
            for line in os.popen('%s --version' % md['redis_bin']).readlines():
                line = line.strip()
                for item in line.split():
                    if '=' in item:
                        key, value = item.split('=')
                        REDIS_SERVER_METADATA[key] = value
            md['redis_server'] = REDIS_SERVER_METADATA
            print('new metadata: %s' % md)
            with open(md_file, 'w') as fh:
                json.dump(md, fh, indent=4)

# Create a dictionary of our arguments, this way this script can be imported
#  without running setup() to allow external scripts to see the setup settings.
args = {
    'package_data': {
        'redislite': ['package_metadata.json', 'bin/redis-server'],
    },
    'include_package_data': True,
    'cmdclass': {
        'build': BuildRedis,
        'install': InstallRedis,
    },

    # We put in a bogus extension module so wheel knows this package has
    # compiled components.
    'ext_modules': [
        Extension('dummy', sources=['src/dummy.c'])
    ],
    # 'extras_require': {
    #     ':sys_platform=="darwin"': [],
    #     ':sys_platform=="linux"': [],
    # }
}
setup_arguments = args

# Add any scripts we want to package
if os.path.isdir('scripts'):
    setup_arguments['scripts'] = [
        os.path.join('scripts', f) for f in os.listdir('scripts')
    ]


class Git(object):
    version_list = ['0', '7', '0']

    def __init__(self, version=None):
        if version:
            self.version_list = version.split('.')

    @property
    def version(self):
        """
        Generate a Unique version value from the git information
        :return:
        """
        git_rev = len(os.popen('git rev-list HEAD').readlines())
        if git_rev != 0:
            self.version_list[-1] = '%d' % git_rev
        version = '.'.join(self.version_list)
        return version

    @property
    def branch(self):
        """
        Get the current git branch
        :return:
        """
        return os.popen('git rev-parse --abbrev-ref HEAD').read().strip()

    @property
    def hash(self):
        """
        Return the git hash for the current build
        :return:
        """
        return os.popen('git rev-parse HEAD').read().strip()

    @property
    def origin(self):
        """
        Return the fetch url for the git origin
        :return:
        """
        for item in os.popen('git remote -v'):
            split_item = item.strip().split()
            if split_item[0] == 'origin' and split_item[-1] == '(push)':
                return split_item[1]


def get_and_update_metadata():
    """
    Get the package metadata or generate it if missing
    :return:
    """
    global METADATA_FILENAME
    global REDIS_SERVER_METADATA

    if not os.path.exists('.git') and os.path.exists(METADATA_FILENAME):
        with open(METADATA_FILENAME) as fh:
            metadata = json.load(fh)
    else:
        git = Git(version=VERSION)
        metadata = {
            'git_origin': git.origin,
            'git_branch': git.branch,
            'git_hash': git.hash,
            'redis_server': REDIS_SERVER_METADATA,
            'redis_bin': install_scripts
        }
        with open(METADATA_FILENAME, 'w') as fh:
            json.dump(metadata, fh, indent=4)
    return metadata


if __name__ == '__main__':
    if sys.platform in UNSUPPORTED_PLATFORMS:
        print(
            'The redislite module is not supported on the %r '
            'platform' % sys.platform,
            file=sys.stderr
        )
        sys.exit(1)

    os.environ['CC'] = 'gcc'

    logging.basicConfig(level=logging.DEBUG)

    if not os.path.exists(REDIS_PATH):
        logger.debug(f'Downloading redis version {REDIS_VERSION}')
        download_redis_submodule()

    logger.debug('Building for platform: %s', distutils.util.get_platform())

    metadata = get_and_update_metadata()

    setup(**setup_arguments)



# Copyright (c) 2015, Yahoo Inc.
# Copyrights licensed under the New BSD License
# See the accompanying LICENSE.txt file for terms.
"""
test_redislite
----------------------------------

Tests for `redislite` module.
"""
from __future__ import print_function
import time
import redislite
import unittest


# noinspection PyPep8Naming
class TestRedisliteSlave(unittest.TestCase):

    def test_slave(self):
        master = redislite.Redis(serverconfig={'port': '7000'})
        slave = redislite.Redis(serverconfig={'slaveof': 'localhost 7000'})
        master.set('key', 'value')
        time.sleep(.5)
        value = slave.get('key').decode(encoding='UTF-8')
        self.assertEquals(value, 'value')
